<doc><ads />
<author>Henry Laxen</author>
<dateWritten>July 3, 2009</dateWritten>
<addtohead>
<link rel="stylesheet" type="text/css" href="/static/js/hscolour.css" />
</addtohead>


<p>This file can be loaded into ghci and executed.  You are encouraged to do so.</p>

> import Prelude hiding ((.), (^), (>), (/), elem, foldl, foldl1)
> import MPS
> import Control.Category hiding ((.))

> hamlet = ["2", "b" , "or", "not", "to", "be"]

<strong>.</strong> is used to pass an argument to the function on the right,
also called <strong>apply</strong> and <strong>send_to</strong> later  

> (.) :: a -> (a -> b) -> b  

<p>Notice that . binds less tightly than function arguments, so
<em>[1..5].drop 2</em> is the same as <em>drop 2 [1..5]</em></p>

> t1 = [1..5].head          -- 1
> t1a = apply [1..5] head   -- 1
> t1b = send_to [1..5] head -- 1
> t2 = [1..5].drop 2        --  [3,4,5]


<p><strong>&gt;</strong> is left to right function composition, read it as <em>then</em></p>

> (>) :: (Category cat) => cat a b -> cat b c -> cat a c
> t3 = tail > length $ [1..5] -- tail then length, is 4

<p><strong>^</strong> is an infix fmap </p>

> (^) :: (Functor f) => f a -> (a -> b) -> f b
> t4 = [1..5] ^ (1+) -- is [2,3,4,5,6]

<p><strong>/</strong> is a FilePath joiner, not division, see System.FilePath.Posix.combine </p>

> (/) :: FilePath -> FilePath -> FilePath
> t5 = "usr" / "local" / "bin" -- is "usr/local/bin" on Linux

<p><strong>join</strong> is intercalate, it inserts the first list, (usually a
String) in between each of the seconds lists, (usually a list of
Strings)</p>

> join :: [a] -> [[a]] -> [a]
> t6 = ["2 b", "to be"].join " or not " -- is "2 b or not to be"

<p><strong>first</strong> .. <strong>tenth</strong> select the corresponding element of a list</p>

> first :: [a] -> a
> t7 = [1..5].third -- is 3

<p><strong>unique</strong> is a synonym for nub, but requires ord.
It returns the unique elements of list  </p> 

> unique :: (Ord a) => [a] -> [a]
> t8 = unique ([1..5] ++ [3..8]) -- is [1,2,3,4,5,6,7,8]

<p><strong>is_unique</strong> returns True if there are no duplicate elements in a list  </p>

> is_unique :: (Ord a) => [a] -> Bool
> t9 = (is_unique [1,2,3,4], is_unique [1,1,2,3]) -- is (True,False)

<p><strong>same</strong> returns True if all the elements of a list are the same  </p>

> same :: (Ord a) => [a] -> Bool
> t10 = same "aaa" -- is True

<p><strong>times</strong> is like an infix replicate, to be used with the new .  </p>

> times :: a -> Int -> [a]
> t11 = 3.times 'a' -- is "aaa"

<p><strong>upto</strong> and <strong>downto</strong> are infix ranges, to be used with the new .  </p>

> upto :: (Enum a) => a -> a -> [a]
> t12 = 1.upto 3 -- is [1,2,3]
> t13 = 3.downto 1 -- is [3,2,1]

<p><strong>remove_at</strong>, <strong>insert_at</strong> and
<strong>replace_at</strong> are all similar, and do their thing at a
particular position of a given list.  </p>

> remove_at :: Int -> [a] -> [a]  
> insert_at, replace_at :: Int -> a -> [a] -> [a]  
> t14 = insert_at  2 'a' "blb" -- is "blab"

<p><strong>at</strong> is list indexing  </p>

> at :: Int -> [a] -> a  
> t15 = at 2 "abcd" -- is 'c'

<p><strong>slice</strong> takes a slice out of a list, if the list is [0..n] then slice a b is</p>

> [x] such that a  <=  x  <  b   
> slice :: Int -> Int -> [a] -> [a]  
> t16 = slice 2 4 [1..5] -- is [3,4] (remember 0 indexing!)

<p><strong>cherry_pick</strong> picks a set of elements out of a list based on a list if indices  </p>

> cherry_pick :: [Int] -> [a] -> [a]
> t17 = cherry_pick [5,0,1] hamlet -- ["be","2","b"]

<p><strong>reduce</strong> and **reduce\'** are synonyms for foldl1, but all the
arguments must be of the same type.  Very useful for chaining
functions together.  The ' version is strict  </p>

> reduce, reduce' :: (a -> a -> a) -> [a] -> a
> t18  = reduce (>)   [(2*), (1+)] 3 -- is 7  (2*3 + 1)
> t19  = reduce (<<<) [(2*), (1+)] 3 -- is 8 (1+3) * 2

<p><strong>inject</strong> and <strong>inject\'</strong> is foldl with the arguments flipped </p>

> inject, inject' :: (Foldable t) => a -> (a -> b -> a) -> t b -> a  
> t20 = [2,3].inject 1 (-) -- is -4 (1-2-3)

<p><strong>none_of</strong> takes a predicate (function a -> Bool) and
a list and returns True if none of the elements of the list satisfy
the predicate. </p>
 
>none_of :: (a -> Bool) -> [a] -> Bool  
> t21 = [1..5].none_of (==6) -- is True

<p><strong>select</strong> and <strong>reject</strong> filter those that match, and filter those that don't match. </p>

 
> select, reject :: (a -> Bool) -> [a] -> [a]  
> t22 = [1..5].reject even -- is [1,3,5]
> t23 = [1..5].select odd  -- is [1,3,5]

<p><strong>inner_map</strong> maps a function over the each element of
an inner list </p>

> inner_map :: (a -> b) -> [[a]] -> [[b]]  
> t24 = inner_map succ hamlet  -- is ["3","c","ps","opu","up","cf"]

<p><strong>inner_reduce</strong> maps reduce over an inner list  </p>

> inner_reduce :: (a -> a -> a) -> [[a]] -> [a]  
> t25 = inner_reduce (+) [[1..5].select odd, [1..5].select even]  -- is [9,6]

<p><strong>inner_inject</strong> maps inject from above over an inner list</p>
  
> inner_inject :: (Foldable t) => a -> (a -> b -> a) -> [t b] -> [a]  
> t26 = inner_inject 1 (-) [[1..5].select odd, [1..5].select even] -- is [-8,-5]

<p><strong>label_by</strong> takes a function and a list, and returns a list of
tuples where the first element of the tuple is f applied to
the element of the list, and the second element is the
original element.  This is an easy way of zipping f of a list
with itself.  </p>

> label_by :: (a -> c) -> [a] -> [(c, a)]  
> t27 =  hamlet.label_by length .take 3 -- is [(1,"2"),(1,"b"),(2,"or")]

<p><strong>labeling</strong> is label_by, with the label coming first  </p>

> labeling :: (a -> c) -> [a] -> [(a, c)]  
> t28 = hamlet.labeling length .take 3 -- is [("2",1),("b",1),("or",2)]

<p><strong>in_group_of</strong> turns a list into a list of lists where the
length of each inner list is given by an Int.  The length of
the last element of the inner list may be less than the Int.</p>

> in_group_of :: Int -> [t] -> [[t]]  
> t29 = join " " hamlet.in_group_of 3   -- is ["2 b"," or"," no","t t","o b","e"]

<p><strong>split_to</strong> turns a list into a list of lists where the number
of sublists is given by an Int.  If the length of the original
list is not divisible by the Int, there will be an additional
element.  </p>

> split_to :: Int -> [a] -> [[a]]  
> t30 = join " " hamlet.split_to 3  -- is ["2 b o","r not"," to b","e"]

<p><strong>belongs_to</strong> and <strong>has</strong> are like elem with different arugments. </p>
 
> belongs_to :: (Foldable t, Eq a) => t a -> a -> Bool  
> has :: (Foldable t, Eq b) => b -> t b -> Bool  
> t31 = "not".belongs_to hamlet -- is True
> t32 = hamlet.has "not" -- is True

<p><strong>indexed</strong> takes a list and returns a list of tuples, the first
element of the tuple being the 0 based index of the position
in the list.  </p>

> indexed :: (Num t, Enum t) => [b] -> [(t, b)]  
> t33 = hamlet.indexed -- is [(0,"2"),(1,"b"),(2,"or"),(3,"not"),(4,"to"),(5,"be")]

<p><strong>map_with_index</strong> first indexes a list, and then applies a map
to the result.  This makes it easy to manipulate each element
of a list based on its position.  </p>

> map_with_index :: (Num t, Enum i) => ((i, a) -> b) -> [a] -> [b]  
> t34 = hamlet.map_with_index (\(x,y) -> if x.even  then y.upper else y)
> -- is ["2","b","OR","not","TO","be"]

<p><strong>ljust</strong> and <strong>rjust</strong> left or right justify a string to a
fixed length.  </p>

> ljust, rjust :: Int -> a -> [a] -> [a]
> t35 = "12".rjust 5 '0'  -- is "00012"
> t36 = "12".ljust 5 '0'  -- is "12000"

<p><strong>swap</strong> exchanges the two elements of a two-tuple</p>

> swap :: (a, b) -> (b, a)  
> t37 = swap (2 , "be") -- is ("be",2)

<p><strong>only_fst</strong> and <strong>only_snd</strong> retrieves
only the first or second element of a list of tuples</p>

> only_fst :: [(a, b)] -> [a]  
> only_snd :: [(a, b)] -> [b]  
> t38 = only_fst (zip hamlet [1..]) -- is ["2","b","or","not","to","be"]
> t39 = only_snd (zip hamlet [1..]) -- is [1,2,3,4,5,6]

<p><strong>map_fst</strong> and <strong>map_snd</strong> maps a
function over only the first or second element of a list of tuples
</p>

> map_fst :: (a -> b) -> [(a, c)] -> [(b, c)]  
> map_snd :: (a -> b) -> [(c, a)] -> [(c, b)]
> t40 = map_fst same (zip hamlet [1..3]) -- is [(True,1),(True,2),(False,3)]
> t41 = map_snd (1+) (zip hamlet [1..3]) -- is [("2",2),("b",3),("or",4)]

<p><strong>filter_fst</strong> and <strong>filter_snd</strong> filters
a list of tuples by looking only at the fst or snd element of each
pair.  </p>

> filter_fst :: (a -> Bool) -> [(a, b)] -> [(a, b)]  
> filter_snd :: (b -> Bool) -> [(a, b)] -> [(a, b)]  
> t42 = filter_fst even (zip [1..] hamlet) -- is [(2,"b"),(4,"not"),(6,"be")]

<p><strong>lower</strong> <strong>upper</strong> and <strong>capitalize</strong> convert a string to lower or upper case  </p>

> lower, upper, capitalize :: String -> String  
> t43 = map capitalize hamlet. join " "    -- is "2 B Or Not To Be"
> t44 = map capitalize > join " " $ hamlet -- is "2 B Or Not To Be"

This file is also available 
<a href="/haskell/MPSexamples.lhs">as an lhs file</a> 
if you want to play with it.  

</doc>
